class PuppetEnterpriseConsole.ResourcesController extends Batman.Controller.WithLayout
  @beforeFilter -> PuppetEnterpriseConsole.set 'active', 'resources'
  @::set('inputs', new Batman.Object)

  # Actions
  index: ->
    @unset 'type'

  show: ({id}) ->
    @set 'type', id
    @set 'klass', Resource[id]
    @set 'inputs.title', ''
    @set 'title', ''

    @set 'sortBy', 'title'
    @set 'resources', @get('klass').get('loadedForSelection')

  details: (params) ->
    type = params.type || Batman.helpers.singularize(params.resource)
    title = params.title || params.id
    return @redirect(type) unless title
    @set 'type', type
    @set 'typeclass', new Batman.UniqueSetIndex(ResourceType.get('loaded'), 'id').get(type)
    @set 'klass', Resource[type]
    @set 'inputs.title', title
    @set 'title', title
    @set 'diffprops', []
    @set 'nodiffprops', []

  # Accessors
  @accessor 'resource', ->
    # TODO: Simplify this.
    klass = @get('klass')

    # this resource accessor should be garbage collected and it's bindings
    # cleaned up when you're done with the resource, but that's not happening
    # So we need to keep from loading the url below when title is undefined
    # If that leak is fixed this line shouldn't be necessary, and network calls
    # to /lm/resources/resource_type/title shouldn't happen on the show action
    # after you've done a detail action
    return unless @get('title')

    obj = new klass(@get('title'))
    obj.get('loadingState').set('loading')
    obj.load (error, data) =>
      obj.get('loadingState').set('loaded')
      unless error
        variations   = data.get("variations")
        nodiff_props = []
        diff_props   = []
        data.get("properties").forEach (p) =>
          has_diff = false
          for v in variations
            prop = JSON.stringify(v.attributes[p]) ? '' unless prop?
            if (prop != JSON.stringify(v.attributes[p]))
              has_diff = true
              diff_props.push(p)
              break
          nodiff_props.push(p) unless has_diff

        @set 'diffprops', diff_props.sort()
        @set 'nodiffprops', nodiff_props.sort()

    return obj

  # Event Handlers
  doSearch: (form) ->
    return unless @get('inputs.title')
    @redirect
      controller: 'resources'
      action:     'details'
      type:       @get('type')
      title:      @get('inputs.title')

  resetSearch: (btn) ->
    return @set('inputs.title', '') unless @get('title')
    @redirect
      controller: 'resources'
      action:     'show'
      id:         @get('type')

  changeSort: (_, sortKey) ->
    if sortKey == @get('sortBy')
      @set 'sortBy', "#{sortKey} desc"
    else
      @set 'sortBy', sortKey

  reloadAll: ->
    Node.selectAll()
    ResourceType.get('all').forEach (type) ->
      Resource[type.get('name')].invalidate().get('forSelection')

  refreshResource: ->
    @get('klass').invalidate().get('forSelection')
