class PuppetEnterpriseConsole.Resource extends Batman.Model
  @global true
  @encode 'title', 'properties', 'variations', 'nodeCount'
  @primaryKey = 'title'
  @storageKey = 'resources'

class PuppetEnterpriseConsole.ResourceType extends Batman.Model
  @global true
  @persist Batman.RestStorage
  @url = "/lm/resource_types"
  @primaryKey = 'name'
  @encode 'name'
  @accessor 'type', -> @get('name')

PuppetEnterpriseConsole.prevent('run')
ResourceType.load (_, types) ->
  for type in types
    Resource[type.get('name')] = class extends Resource
      @__name__ = type.get('name')
      @set 'loadingState', new StateMachine()

      constructor: ->
        super
        @set 'loadingState', new StateMachine()

      @persist(Batman.RestStorage.Filtered.ByNodeSet).after 'readAll', (args) =>
        @set('rawData', args[2])
        args

      PuppetEnterpriseConsole.on 'ready', =>
        Node.observe 'nodeSetID', =>
          @invalidate()

      loadedSet = new Batman.Set()
      loadedSet.accessor 'timeago', ->
        Timer.get('everyMinute')
        jQuery.timeago(@get('timestamp')) if @get('timestamp')?

      @set('loadedForSelection', loadedSet)

      @classAccessor 'forSelection'
        get: ->
          resources = @get('loadedForSelection')
          unless resources.get('state')
            resources.set('state', 'loading')
            resources.unset('total')
            resources.unset('variations')
            resources.unset('unique')
            resources.clear()
            @load (err, nodes) =>
              resources.add(nodes...)
              resources.set('state', 'loaded')

              now = new Date()
              resources.set 'humanTime', "#{now.toLocaleDateString()} at #{now.toLocaleTimeString()}"
              resources.set 'timestamp', now

              resources.set 'variance', Batman.Filters.asPercentOf(resources.variations - resources.unique, resources.total - resources.unique)

          return resources

      @classAccessor 'rawData'
        set: (k, v) ->
          @get('forSelection').set('total', v.total)
          @get('forSelection').set('variations', v.variations)
          @get('forSelection').set('unique', v.resources.length)

      @invalidate: ->
        @get('loaded').clear()
        @get('loadedForSelection').clear()
        @get('loadedForSelection').unset('state')
        @get('loadedForSelection').unset('total')
        @get('loadedForSelection').unset('humanTime')
        @get('loadedForSelection').unset('timestamp')
        @get('loadedForSelection').unset('variance')
        @get('loadedForSelection').unset('name')
        return this

      @url: "/lm/resources/#{type.get('name')}"
      url: -> "#{@constructor.url}/#{@get('title')}"

      @accessor 'isoTimestamp', ->
        Timer.get('everyMinute')
        return @get('timestamp')?.toISOString()

    PuppetEnterpriseConsole.route "#{type.get('name')}/:id", 'resources#details'
      resource: Batman.helpers.pluralize(type.get('name'))
      action: 'details'
    PuppetEnterpriseConsole.route PuppetEnterpriseConsole.dispatcher

  PuppetEnterpriseConsole.allow('run')
  PuppetEnterpriseConsole.run() if PuppetEnterpriseConsole.wantsToRun
